@@ -14,8 +14,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import uuid
+from datetime import datetime
 
+from oslo_db.exception import DBDuplicateEntry
 from oslo_log import log as logging
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
@@ -23,10 +24,12 @@ from oslo_utils import uuidutils
 import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy.orm import exc as orm_exc
+from sqlalchemy import schema
 
 from tacker.api.v1 import attributes
+from tacker.common import exceptions
 from tacker import context as t_context
-from tacker.db.common_services import common_services_db
+from tacker.db.common_services import common_services_db_plugin
 from tacker.db import db_base
 from tacker.db import model_base
 from tacker.db import models_v1
@@ -57,10 +60,10 @@ class VNFD(model_base.BASE, models_v1.HasId, models_v1.HasTenant,
 
     # service type that this service vm provides.
     # At first phase, this includes only single service
-    # In future, single service VM may accomodate multiple services.
+    # In future, single service VM may accommodate multiple services.
     service_types = orm.relationship('ServiceType', backref='vnfd')
 
-    # driver to communicate with service managment
+    # driver to communicate with service management
     mgmt_driver = sa.Column(sa.String(255))
 
     # (key, value) pair to spin up
@@ -70,6 +73,14 @@ class VNFD(model_base.BASE, models_v1.HasId, models_v1.HasTenant,
     # vnfd template source - inline or onboarded
     template_source = sa.Column(sa.String(255), server_default='onboarded')
 
+    __table_args__ = (
+        schema.UniqueConstraint(
+            "tenant_id",
+            "name",
+            "deleted_at",
+            name="uniq_vnfd0tenant_id0name0deleted_at"),
+    )
+
 
 class ServiceType(model_base.BASE, models_v1.HasId, models_v1.HasTenant):
     """Represents service type which hosting vnf provides.
@@ -127,6 +138,14 @@ class VNF(model_base.BASE, models_v1.HasId, models_v1.HasTenant,
     vim = orm.relationship('Vim')
     error_reason = sa.Column(sa.Text, nullable=True)
 
+    __table_args__ = (
+        schema.UniqueConstraint(
+            "tenant_id",
+            "name",
+            "deleted_at",
+            name="uniq_vnf0tenant_id0name0deleted_at"),
+    )
+
 
 class VNFAttribute(model_base.BASE, models_v1.HasId):
     """Represents kwargs necessary for spinning up VM in (key, value) pair.
@@ -156,7 +175,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
 
     def __init__(self):
         super(VNFMPluginDb, self).__init__()
-        self._cos_db_plg = common_services_db.CommonServicesPluginDb()
+        self._cos_db_plg = common_services_db_plugin.CommonServicesPluginDb()
 
     def _get_resource(self, context, model, id):
         try:
@@ -196,8 +215,8 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
         return dict((arg.key, arg.value) for arg in dev_attrs_db)
 
     def _make_vnf_dict(self, vnf_db, fields=None):
-        LOG.debug(_('vnf_db %s'), vnf_db)
-        LOG.debug(_('vnf_db attributes %s'), vnf_db.attributes)
+        LOG.debug('vnf_db %s', vnf_db)
+        LOG.debug('vnf_db attributes %s', vnf_db.attributes)
         res = {
             'vnfd':
             self._make_vnfd_dict(vnf_db.vnfd),
@@ -219,47 +238,52 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
 
     def create_vnfd(self, context, vnfd):
         vnfd = vnfd['vnfd']
-        LOG.debug(_('vnfd %s'), vnfd)
+        LOG.debug('vnfd %s', vnfd)
         tenant_id = self._get_tenant_id_for_create(context, vnfd)
         service_types = vnfd.get('service_types')
         mgmt_driver = vnfd.get('mgmt_driver')
         template_source = vnfd.get("template_source")
 
         if (not attributes.is_attr_set(service_types)):
-            LOG.debug(_('service types unspecified'))
+            LOG.debug('service types unspecified')
             raise vnfm.ServiceTypesNotSpecified()
 
-        with context.session.begin(subtransactions=True):
-            vnfd_id = str(uuid.uuid4())
-            vnfd_db = VNFD(
-                id=vnfd_id,
-                tenant_id=tenant_id,
-                name=vnfd.get('name'),
-                description=vnfd.get('description'),
-                mgmt_driver=mgmt_driver,
-                template_source=template_source)
-            context.session.add(vnfd_db)
-            for (key, value) in vnfd.get('attributes', {}).items():
-                attribute_db = VNFDAttribute(
-                    id=str(uuid.uuid4()),
-                    vnfd_id=vnfd_id,
-                    key=key,
-                    value=value)
-                context.session.add(attribute_db)
-            for service_type in (item['service_type']
-                                 for item in vnfd['service_types']):
-                service_type_db = ServiceType(
-                    id=str(uuid.uuid4()),
+        try:
+            with context.session.begin(subtransactions=True):
+                vnfd_id = uuidutils.generate_uuid()
+                vnfd_db = VNFD(
+                    id=vnfd_id,
                     tenant_id=tenant_id,
-                    vnfd_id=vnfd_id,
-                    service_type=service_type)
-                context.session.add(service_type_db)
-
-        LOG.debug(_('vnfd_db %(vnfd_db)s %(attributes)s '),
+                    name=vnfd.get('name'),
+                    description=vnfd.get('description'),
+                    mgmt_driver=mgmt_driver,
+                    template_source=template_source,
+                    deleted_at=datetime.min)
+                context.session.add(vnfd_db)
+                for (key, value) in vnfd.get('attributes', {}).items():
+                    attribute_db = VNFDAttribute(
+                        id=uuidutils.generate_uuid(),
+                        vnfd_id=vnfd_id,
+                        key=key,
+                        value=value)
+                    context.session.add(attribute_db)
+                for service_type in (item['service_type']
+                                     for item in vnfd['service_types']):
+                    service_type_db = ServiceType(
+                        id=uuidutils.generate_uuid(),
+                        tenant_id=tenant_id,
+                        vnfd_id=vnfd_id,
+                        service_type=service_type)
+                    context.session.add(service_type_db)
+        except DBDuplicateEntry as e:
+            raise exceptions.DuplicateEntity(
+                _type="vnfd",
+                entry=e.columns)
+        LOG.debug('vnfd_db %(vnfd_db)s %(attributes)s ',
                   {'vnfd_db': vnfd_db,
                    'attributes': vnfd_db.attributes})
         vnfd_dict = self._make_vnfd_dict(vnfd_db)
-        LOG.debug(_('vnfd_dict %s'), vnfd_dict)
+        LOG.debug('vnfd_dict %s', vnfd_dict)
         self._cos_db_plg.create_event(
             context, res_id=vnfd_dict['id'],
             res_type=constants.RES_TYPE_VNFD,
@@ -327,7 +351,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
     def choose_vnfd(self, context, service_type,
                     required_attributes=None):
         required_attributes = required_attributes or []
-        LOG.debug(_('required_attributes %s'), required_attributes)
+        LOG.debug('required_attributes %s', required_attributes)
         with context.session.begin(subtransactions=True):
             query = (
                 context.session.query(VNFD).
@@ -343,7 +367,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
                         VNFD.id ==
                         VNFDAttribute.vnfd_id,
                         VNFDAttribute.key == key)))
-            LOG.debug(_('statements %s'), query)
+            LOG.debug('statements %s', query)
             vnfd_db = query.first()
             if vnfd_db:
                 return self._make_vnfd_dict(vnfd_db)
@@ -357,39 +381,45 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
             arg.value = value
         else:
             arg = VNFAttribute(
-                id=str(uuid.uuid4()), vnf_id=vnf_id,
+                id=uuidutils.generate_uuid(), vnf_id=vnf_id,
                 key=key, value=value)
             context.session.add(arg)
 
     # called internally, not by REST API
     def _create_vnf_pre(self, context, vnf):
-        LOG.debug(_('vnf %s'), vnf)
+        LOG.debug('vnf %s', vnf)
         tenant_id = self._get_tenant_id_for_create(context, vnf)
         vnfd_id = vnf['vnfd_id']
         name = vnf.get('name')
-        vnf_id = str(uuid.uuid4())
+        vnf_id = uuidutils.generate_uuid()
         attributes = vnf.get('attributes', {})
         vim_id = vnf.get('vim_id')
         placement_attr = vnf.get('placement_attr', {})
-        with context.session.begin(subtransactions=True):
-            vnfd_db = self._get_resource(context, VNFD,
-                                         vnfd_id)
-            vnf_db = VNF(id=vnf_id,
-                         tenant_id=tenant_id,
-                         name=name,
-                         description=vnfd_db.description,
-                         instance_id=None,
-                         vnfd_id=vnfd_id,
-                         vim_id=vim_id,
-                         placement_attr=placement_attr,
-                         status=constants.PENDING_CREATE,
-                         error_reason=None)
-            context.session.add(vnf_db)
-            for key, value in attributes.items():
-                    arg = VNFAttribute(
-                        id=str(uuid.uuid4()), vnf_id=vnf_id,
-                        key=key, value=value)
-                    context.session.add(arg)
+        try:
+            with context.session.begin(subtransactions=True):
+                vnfd_db = self._get_resource(context, VNFD,
+                                             vnfd_id)
+                vnf_db = VNF(id=vnf_id,
+                             tenant_id=tenant_id,
+                             name=name,
+                             description=vnfd_db.description,
+                             instance_id=None,
+                             vnfd_id=vnfd_id,
+                             vim_id=vim_id,
+                             placement_attr=placement_attr,
+                             status=constants.PENDING_CREATE,
+                             error_reason=None,
+                             deleted_at=datetime.min)
+                context.session.add(vnf_db)
+                for key, value in attributes.items():
+                        arg = VNFAttribute(
+                            id=uuidutils.generate_uuid(), vnf_id=vnf_id,
+                            key=key, value=value)
+                        context.session.add(arg)
+        except DBDuplicateEntry as e:
+            raise exceptions.DuplicateEntity(
+                _type="vnf",
+                entry=e.columns)
         evt_details = "VNF UUID assigned."
         self._cos_db_plg.create_event(
             context, res_id=vnf_id,
@@ -404,7 +434,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
     # intsance_id = None means error on creation
     def _create_vnf_post(self, context, vnf_id, instance_id,
                          mgmt_url, vnf_dict):
-        LOG.debug(_('vnf_dict %s'), vnf_dict)
+        LOG.debug('vnf_dict %s', vnf_dict)
         with context.session.begin(subtransactions=True):
             query = (self._model_query(context, VNF).
                      filter(VNF.id == vnf_id).
@@ -489,13 +519,14 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
         return updated_vnf_dict
 
     def _update_vnf_post(self, context, vnf_id, new_status,
-                         new_vnf_dict=None):
+                         new_vnf_dict):
+        updated_time_stamp = timeutils.utcnow()
         with context.session.begin(subtransactions=True):
             (self._model_query(context, VNF).
              filter(VNF.id == vnf_id).
              filter(VNF.status == constants.PENDING_UPDATE).
              update({'status': new_status,
-                     'updated_at': timeutils.utcnow()}))
+                     'updated_at': updated_time_stamp}))
 
             dev_attrs = new_vnf_dict.get('attributes', {})
             (context.session.query(VNFAttribute).
@@ -510,9 +541,9 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
         self._cos_db_plg.create_event(
             context, res_id=vnf_id,
             res_type=constants.RES_TYPE_VNF,
-            res_state=new_vnf_dict['status'],
+            res_state=new_status,
             evt_type=constants.RES_EVT_UPDATE,
-            tstamp=new_vnf_dict[constants.RES_EVT_UPDATED_FLD])
+            tstamp=updated_time_stamp)
 
     def _delete_vnf_pre(self, context, vnf_id):
         with context.session.begin(subtransactions=True):
@@ -570,7 +601,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
         # start actual creation of hosting vnf.
         # Waiting for completion of creation should be done backgroundly
         # by another thread if it takes a while.
-        instance_id = str(uuid.uuid4())
+        instance_id = uuidutils.generate_uuid()
         vnf_dict['instance_id'] = instance_id
         self._create_vnf_post(context, vnf_dict['id'], instance_id, None,
                               vnf_dict)
@@ -584,7 +615,9 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
         # start actual update of hosting vnf
         # waiting for completion of update should be done backgroundly
         # by another thread if it takes a while
-        self._update_vnf_post(context, vnf_id, constants.ACTIVE)
+        self._update_vnf_post(context, vnf_id,
+                              constants.ACTIVE,
+                              vnf_dict)
         return vnf_dict
 
     # reference implementation. needs to be overrided by subclass
@@ -622,7 +655,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
                     filter(~VNF.status.in_(exclude_status)).
                     with_lockmode('update').one())
             except orm_exc.NoResultFound:
-                LOG.warning(_('no vnf found %s'), vnf_id)
+                LOG.warning('no vnf found %s', vnf_id)
                 return False
 
             vnf_db.update({'status': new_status})
