@@ -14,14 +14,18 @@
 #    under the License.
 
 import codecs
+from datetime import datetime
 import os
 import yaml
 
 
+DUMMY_NS_2_NAME = 'dummy_ns_2'
+
+
 def _get_template(name):
     filename = os.path.join(
         os.path.dirname(os.path.abspath(__file__)),
-        "../vm/infra_drivers/openstack/data/", name)
+        "../vnfm/infra_drivers/openstack/data/", name)
     f = codecs.open(filename, encoding='utf-8', errors='strict')
     return f.read()
 
@@ -29,18 +33,28 @@ tosca_vnfd_openwrt = _get_template('test_tosca_openwrt.yaml')
 config_data = _get_template('config_data.yaml')
 update_config_data = _get_template('update_config_data.yaml')
 vnffg_params = _get_template('vnffg_params.yaml')
-vnffgd_template = yaml.load(_get_template('vnffgd_template.yaml'))
-vnffgd_tosca_template = yaml.load(_get_template('tosca_vnffgd_template.yaml'))
-vnffgd_tosca_param_template = yaml.load(_get_template(
+vnffg_multi_params = _get_template('vnffg_multi_params.yaml')
+vnffgd_template = yaml.safe_load(_get_template('vnffgd_template.yaml'))
+vnffgd_tosca_template = yaml.safe_load(_get_template(
+    'tosca_vnffgd_template.yaml'))
+vnffgd_tosca_param_template = yaml.safe_load(_get_template(
     'tosca_vnffgd_param_template.yaml'))
-vnffgd_invalid_tosca_template = yaml.load(_get_template(
+vnffgd_tosca_str_param_template = yaml.safe_load(_get_template(
+    'tosca_vnffgd_str_param_template.yaml'))
+vnffgd_tosca_multi_param_template = yaml.safe_load(_get_template(
+    'tosca_vnffgd_multi_param_template.yaml'))
+vnffgd_invalid_tosca_template = yaml.safe_load(_get_template(
     'tosca_invalid_vnffgd_template.yaml'))
 vnfd_scale_tosca_template = _get_template('tosca_scale.yaml')
 vnfd_alarm_respawn_tosca_template = _get_template(
     'test_tosca_vnfd_alarm_respawn.yaml')
 vnfd_alarm_scale_tosca_template = _get_template(
     'test_tosca_vnfd_alarm_scale.yaml')
-nsd_tosca_template = yaml.load(_get_template('tosca_nsd_template.yaml'))
+vnfd_alarm_multi_actions_tosca_template = _get_template(
+    'test_tosca_vnfd_alarm_multi_actions.yaml')
+nsd_tosca_template = yaml.safe_load(_get_template('tosca_nsd_template.yaml'))
+vnffgd_wrong_cp_number_template = yaml.safe_load(_get_template(
+    'tosca_vnffgd_wrong_cp_number_template.yaml'))
 
 
 def get_dummy_vnfd_obj():
@@ -82,6 +96,7 @@ def get_dummy_vnf_obj():
                     'vim_id': u'6261579e-d6f3-49ad-8bc3-a9cb974778ff',
                     'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
                     'name': 'dummy_vnf',
+                    'deleted_at': datetime.min,
                     'attributes': {},
                     'vnfd_template': None}}
 
@@ -152,14 +167,23 @@ def get_vim_auth_obj():
 
 
 def get_dummy_vnffgd_obj():
-    return {u'vnffgd': {'name': 'dummy_vnfd',
+    return {u'vnffgd': {'name': 'dummy_vnffgd',
                         'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
                         u'template': {u'vnffgd': vnffgd_tosca_template},
-                        'description': 'dummy_vnfd_description'}}
+                        'description': 'dummy_vnffgd_description',
+                        'template_source': 'onboarded'}}
+
+
+def get_dummy_vnffgd_obj_inline():
+    return {u'vnffgd': {'name': 'dummy_vnffgd_inline',
+                        'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
+                        u'template': {u'vnffgd': vnffgd_tosca_template},
+                        'description': 'dummy_vnffgd_description_inline',
+                        'template_source': 'inline'}}
 
 
 def get_dummy_vnffg_obj():
-    return {'vnffg': {'description': 'dummy_vnf_description',
+    return {'vnffg': {'description': 'dummy_vnffg_description',
                       'vnffgd_id': u'eb094833-995e-49f0-a047-dfb56aaf7c4e',
                       'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
                       'name': 'dummy_vnffg',
@@ -168,6 +192,16 @@ def get_dummy_vnffg_obj():
                       'symmetrical': False}}
 
 
+def get_dummy_vnffg_obj_inline():
+    return {'vnffg': {'description': 'dummy_vnffg_description_inline',
+                      'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
+                      'name': 'dummy_vnffg_inline',
+                      u'attributes': {u'template': vnffgd_tosca_template},
+                      'vnf_mapping': {},
+                      'symmetrical': False,
+                      'vnffgd_template': vnffgd_tosca_template}}
+
+
 def get_dummy_vnffg_param_obj():
     return {'vnffg': {'description': 'dummy_vnf_description',
                       'vnffgd_id': u'eb094833-995e-49f0-a047-dfb56aaf7c4e',
@@ -181,6 +215,32 @@ def get_dummy_vnffg_param_obj():
                       'symmetrical': False}}
 
 
+def get_dummy_vnffg_str_param_obj():
+    return {'vnffg': {'description': 'dummy_vnf_description',
+                      'vnffgd_id': u'eb094833-995e-49f0-a047-dfb56aaf7c4e',
+                      'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
+                      'name': 'dummy_vnffg',
+                      u'attributes': {
+                          u'template': vnffgd_tosca_param_template},
+                      'vnf_mapping': {},
+                      u'attributes': {
+                          u'param_values': 'value not dict format'},
+                      'symmetrical': False}}
+
+
+def get_dummy_vnffg_multi_param_obj():
+    return {'vnffg': {'description': 'dummy_vnf_description',
+                      'vnffgd_id': u'eb094833-995e-49f0-a047-dfb56aaf7c4e',
+                      'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
+                      'name': 'dummy_vnffg',
+                      u'attributes': {
+                          u'template': vnffgd_tosca_multi_param_template},
+                      'vnf_mapping': {},
+                      u'attributes': {u'param_values':
+                          yaml.safe_load(vnffg_multi_params)},
+                      'symmetrical': False}}
+
+
 def get_dummy_vnffg_obj_vnf_mapping():
     return {'vnffg': {'description': 'dummy_vnf_description',
                       'vnffgd_id': u'eb094833-995e-49f0-a047-dfb56aaf7c4e',
@@ -195,11 +255,19 @@ def get_dummy_vnffg_obj_vnf_mapping():
 
 
 def get_dummy_nsd_obj():
-    return {'nsd': {'description': 'dummy nsd description',
+    return {'nsd': {'description': 'dummy_nsd_description',
                     'name': 'dummy_NSD',
                     'tenant_id': u'8819a1542a5948b68f94d4be0fd50496',
-                    'template': {},
-                    'attributes': {u'nsd': nsd_tosca_template}}}
+                    'attributes': {u'nsd': nsd_tosca_template},
+                    'template_source': 'onboarded'}}
+
+
+def get_dummy_nsd_obj_inline():
+    return {'nsd': {'description': 'dummy_nsd_description_inline',
+                    'name': 'dummy_NSD_inline',
+                    'tenant_id': u'8819a1542a5948b68f94d4be0fd50496',
+                    'attributes': {u'nsd': nsd_tosca_template},
+                    'template_source': 'inline'}}
 
 
 def get_dummy_ns_obj():
@@ -212,3 +280,27 @@ def get_dummy_ns_obj():
                    'attributes': {
                        'param_values': {'nsd': {'vl1_name': 'net_mgmt',
                                                 'vl2_name': 'net0'}}}}}
+
+
+def get_dummy_ns_obj_inline():
+    return {'ns': {'description': 'dummy_ns_description_inline',
+                   'id': u'ff35e3f0-0a11-4071-bce6-279fdf1c8bf9',
+                   'vim_id': u'6261579e-d6f3-49ad-8bc3-a9cb974778ff',
+                   'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
+                   'name': 'dummy_ns_inline',
+                   'attributes': {
+                       'param_values': {'nsd': {'vl1_name': 'net_mgmt',
+                                                'vl2_name': 'net0'}}},
+                   'nsd_template': nsd_tosca_template}}
+
+
+def get_dummy_ns_obj_2():
+    return {'ns': {'description': 'dummy_ns_description',
+                   'id': u'ba6bf017-f6f7-45f1-a280-57b073bf78ea',
+                   'nsd_id': u'eb094833-995e-49f0-a047-dfb56aaf7c4e',
+                   'vim_id': u'6261579e-d6f3-49ad-8bc3-a9cb974778ff',
+                   'tenant_id': u'ad7ebc56538745a08ef7c5e97f8bd437',
+                   'name': DUMMY_NS_2_NAME,
+                   'attributes': {
+                       'param_values': {'nsd': {'vl1_name': 'net_mgmt',
+                                                'vl2_name': 'net0'}}}}}
